/*
ULDC hub emblems
	by Lach

v1.1
*/

// IMPORTANT INFO
// 1. Make sure this script loads after OLDC/Globals.lua!

// 2. Hub emblems that are collected are saved using I/O.
// The file is stored in luafiles/ULDC and the name is generated using the settings from OLDC/Globals.lua (hence the previous point).

// 3. There's currently no way to read emblem data in Lua, so we have to hardcode (hardscript?) which skin-based emblems exist in the map.
// If you change the emblem tags/skins (or use this script in a future hub), you need to update this table.
// Once you release the addon, avoid changing this table again unless you also invalidate the gamedata, as it is used to format the I/O file.
// (also I don't foresee that this will ever be a problem but only up to 32 emblems are supported atm)
local HUB_EMBLEMS = {
	{
		skin = "sonic",
		tag = 0
	},
	{
		skin = "tails",
		tag = 1
	},
	{
		skin = "knuckles",
		tag = 2
	},
	{
		skin = "amy",
		tag = 3
	},
	{
		skin = "fang",
		tag = 4
	},
	{
		skin = "metalsonic",
		tag = 5
	}
}

// (you don't need to edit anything past this point (unless you want to))

local EMBLEM_THINGNUM = mobjinfo[MT_EMBLEM].doomednum
local FILE_NAME = "ULDC/ULDCAutumn2021.dat"

local prevSkin = -1
local emblemsActive = false

local saveBits = 0

local loaded = false

local function Load()
	if loaded
		return
	end
	
	loaded = true
	
	local file = io.openlocal(FILE_NAME)
	if file
		saveBits = file:read("*n") or 0
		file:close()
	end
end

local function Save()
	local file = io.openlocal(FILE_NAME, 'w')
	file:write(saveBits)
	file:close()
end

addHook("MapChange", do
	emblemsActive = false
	prevSkin = consoleplayer.skin
end)

addHook("MapLoad", do
	if gamemap ~= OLDC.hub
	or multiplayer or netgame // emblems don't spawn in multiplayer (yet, but this script only accounts for singleplayer anyway, so a lot will need to be updated if this changes)
		return
	end
	
	Load()
	
	for mt in mapthings.iterate
		if mt.type ~= EMBLEM_THINGNUM
		or not mt.mobj
			continue
		end
		
		// this mapthing spawned an emblem, which means emblems are active
		emblemsActive = true
		
		// attempt to detect whether the gamedata was reset
		// this will only work if you load up a save as a character you'd collected the emblem for in the previous gamedata
		// but it's worth doing I guess
		for i, data in ipairs(HUB_EMBLEMS)
			if data.tag ~= mt.angle
			or not (saveBits & (1 << (i - 1))) // if the emblem was never collected, it can't be used to detect a reset gamedata
				continue
			end
			
			if mt.mobj.valid and mt.mobj.health > 0
			and mt.mobj.flags & MF_SPECIAL
				saveBits = 0
				Save()
				break
			end
		end
		
		if saveBits == 0
			break
		end
	end
end)

addHook("MobjDeath", function(mo, inf, source)
	if not emblemsActive
		return
	end
	
	if inf ~= nil // P_TouchSpecialThing does not pass an inflictor, so if one exists, this emblem was killed via another means
	or not (source and source.valid and source.player) // ensure emblem was picked up by a player
		return
	end
	
	for mt in mapthings.iterate
		if mt.type ~= EMBLEM_THINGNUM
		or mt.mobj ~= mo
			continue
		end
		
		for i, data in ipairs(HUB_EMBLEMS)
			if data.tag ~= mt.angle
				continue
			end
			
			saveBits = $ | (1 << (i - 1))
			Save()
		end
	end
end, MT_EMBLEM)

addHook("ThinkFrame", do
	if not emblemsActive
	or titlemapinaction
	or consoleplayer.skin == prevSkin
		return
	end

	local skinName = skins[consoleplayer.skin].name

	for mt in mapthings.iterate
		if mt.type ~= EMBLEM_THINGNUM
			continue
		end

		local mo = mt.mobj

		if not (mo.valid and mo.health > 0)
			continue
		end

		for i, data in ipairs(HUB_EMBLEMS)
			if data.tag ~= mt.angle
			or saveBits & (1 << (i - 1))
				continue
			end

			if data.skin == skinName
				A_SetObjectFlags(mo, MF_NOBLOCKMAP, 1)
				mo.frame = $ & ~FF_TRANSMASK
				mo.flags = $ & ~(MF_NOCLIP|MF_NOCLIPTHING) | MF_SPECIAL
			else
				//A_SetObjectFlags(mo, MF_NOBLOCKMAP, 2) // this sometimes causes the game to crash?? so we're gonna use MF_NOCLIPTHING instead
				mo.frame = $ & ~FF_TRANSMASK | FF_TRANS50
				mo.flags = $ & ~MF_SPECIAL | MF_NOCLIP | MF_NOCLIPTHING
			end
		end
	end

	prevSkin = consoleplayer.skin
end)